// This code estimates a QES demand system
// j indexes goods j=1,..,J, t indexes demographics t=1,...,T
// qbarj are group-average spending on each of J goods (e.g., qbar1, qbar2, qbar3 for 3 goods)
// i indexes observations, k indexes other observations in the group
// zt are T demographic shifters (e.g., z1,z2,z3,z4 for 4 characteristics---the first should be a constant)
// qij,xi,zit and qkj,xk,zkt are pairs of observations within group
// 		ensure that all zt are z-scored or otherwise have similar scales
// qikbarj are leave-two-out group averages
// xbarlag is the average x in the same group in other periods, and xbar2lag is its square; x2barlag is the average x2;
//    letting t=1,...,T: ztbarlag is the average zt in other periods; xztbarlag is the average of x*zt in other periods
// note that qi and qk are actually quantities, and not spending
// note that x and x2 are nominal, but xbar etc are average real expenditures from other periods

/******************************************************************************
 Proceed in 4 parts:
 
 1/ adjust data
 2/ construct moment equations
 3/ make instruments
 4/ estimate
******************************************************************************/

set seed 339487731

local J=J
local Jm1=`J'-1
local T=$T
local Tm1=`T'-1
local Tindiv : word count $zlist_indiv

/******************************************************************************
Set weights
******************************************************************************/

tempvar clust_weight num_group_round
bys group_round: egen `num_group_round'=count(group_round)
g double `clust_weight'=size_group_round/`num_group_round'
if $meas_pc==1 replace `clust_weight' = `clust_weight'*hhsizei*hhsizek
global GMM_weight "[aweight=`clust_weight']"

/******************************************************************************
 Update data for J=2, and J=3
******************************************************************************/

if `J'==2 {
	replace p1=P1
	replace p2=P2
	replace qi1=Qi1
	replace qk1=Qk1
	replace qikbar1=Qikbar1
	replace qi2=Qi2
	replace qk2=Qk2
	replace qikbar2=Qikbar2
	
	foreach var of varlist qi3 qi4 qk3 qk4 p3 p4 qikbar3 qikbar4 {
		qui replace `var'=.
	}
}

if `J'==3 {
	forv j = 1(1)3 {
		replace p`j'=P`j'
		replace qi`j'=Qi`j'
		replace qk`j'=Qk`j'
		replace qikbar`j'=Qikbar`j'
	}
	
	foreach var of varlist qi4 qk4 p4 qikbar4 {
		qui replace `var'=.
	}
}


/******************************************************************************
 Make moment equations
******************************************************************************/

global zilist ""
global zklist ""
forval t=1/`Tm1' {
	global zilist "$zilist zi`t'"
	global zklist "$zklist zk`t'"
}
global zilistTm1 "$zilist"
global zklistTm1 "$zklist"
if `T'>0 global zilist "$zilist zi`T'"
if `T'>0 global zklist "$zklist zk`T'"

// create b_p and pC
// remember b_p doesn't vary within group
local b_p "exp( 0"
forval j=1/`Jm1' {
	local b_p "`b_p' + {b`j'}*ln(p`j'/p`J')"
}
local b_p "`b_p' + ln(p`J') )"
if simple_model==1	local b_p "1"
di "`b_p'"

// pCj is the sum of these two, for use in the RE model
// pCij, pCkj are the individually varying parts of Cz*pj, for use in the FE model
// pCgj is the part of Cz*pj that does not vary within group, for use in the FE model
forval j=1/`J' {
	local pC`j' "(p`j'*{C`j':$zilist})"
}
local Tindivp1=`Tindiv'+1
forval j=1/`J' {
	local pCi`j' "( 0 "
	local pCk`j' "( 0 "
	local pCg`j' "( 0 "
	forval t=1/`Tindiv' {
		local pCi`j' "`pCi`j'' + p`j'*{C`j'`t'}*zi`t' "
		local pCk`j' "`pCk`j'' + p`j'*{C`j'`t'}*zk`t' "
	}
	forval t=`Tindivp1'/`T' {
		local pCg`j' "`pCg`j'' + p`j'*{C`j'`t'}*zi`t' "
	}
	local pCi`j' "`pCi`j'' )"
	local pCk`j' "`pCk`j'' )"
	local pCg`j' "`pCg`j'' )"
}

// pC, pCi, pCk, pCg are the sums of the j's computed above, and including the J'th element
local pC "( 0"
local pCi "( 0"
local pCk "( 0"
local pCg "( 0"
forval j=1/`J' {
	local pC "`pC' + `pC`j''"
	local pCi "`pCi' + `pCi`j''"
	local pCk "`pCk' + `pCk`j''"
	local pCg "`pCg' + `pCg`j''"
}
local pC "`pC' )"
local pCi "`pCi' )"
local pCk "`pCk' )"
local pCg "`pCg' )"

// the RE model needs versions of pC and pCj for 2nd and subsequent occurences in the GMM statement
local pC_2nd "( 0"
forval j=1/`J' {
	local pC`j' "(p`j'*{C`j':})"
	local pC_2nd "`pC_2nd' + `pC`j''"
}
local pC_2nd "`pC_2nd' )"

// make price terms:  	budget shifter is rp'Drp, where rp is the root-price vector
//  					expenditure shifter is p*diag(D) + (1/2) Sum _ s~j rp'Drp
// price terms do NOT include their diagonal elements, because these are picked up by the constant term in z (zT)
local rpDrp "( 0"
forval j=1/`J' {
	local r=`j'+1
	local m=`j'-1
	forval s=`r'/`J' {
		if `j'==`s' local rpDrp "`rpDrp' + {D`j'`s'}*sqrt(p`j')*sqrt(p`s')*2"
	}
	local Drp`j' "( 0"
	forval s=1/`m' {
		if `j'==`s' local Drp`j' "`Drp`j'' + {D`s'`j'}*sqrt(p`s')*sqrt(p`j')"
	}
	forval s=`r'/`J' {
		if `j'==`s' local Drp`j' "`Drp`j'' + {D`j'`s'}*sqrt(p`j')*sqrt(p`s')"
	}
	
	local Drp`j' "`Drp`j'' )"
	di "`Drp`j''"
}

local rpDrp "`rpDrp' )"
di "`rpDrp'"

if simple_model==1	{
	local pC "0"
	local pC1 "0"
	local pCi "0"
	local pCk "0"
	local pC_2nd "0"
	local rpDrp "0"
	local Drp1 "0"
}	

// create Aq 
// note that ikbar means group-level average excluding observations i and k, for use in FE and RE
// k means use qk as the group quantity (for use in RE only)
local qtypelist "ikbar k bar"
foreach qtype of local qtypelist {
	if Afull==1 {
		forval j=1/`J' {
			local A`qtype'`j' "( 0"
			
			forval l=1/`J' {
				if same_spillover==0            local A_term "{A`j'`l':}"
				if same_spillover==1 & `j'==`l' local A_term "{A11:}"
				if same_spillover==1 & `j'!=`l' local A_term "{A12:}"
			
				local A`qtype'`j' "`A`qtype'`j'' + `A_term'*p`j'*q`qtype'`l'"
			}
			local A`qtype'`j' "`A`qtype'`j'' )"
		}
	}

	if Adiag==1 {
		forval j=1/`J' {
			if same_spillover==0 local A_term "{A`j'`j':}"
			if same_spillover==1 local A_term "{A11:}"
			
			local A`qtype'`j' "(`A_term'*p`j'*q`qtype'`j')"
		}
	}
	
	local A`qtype' "( 0"
	forval j=1/`J' {
		local A`qtype' "`A`qtype'' + `A`qtype'`j''"
	}
	local A`qtype' "`A`qtype'' )"
}

// Aikbar is A'qbar-minus-ik; Ak is A'qk
// x_hat, x_hat_2nd are  x-pCz (full z), used in RE
// xi_hat and xk_hat are x minus the individually varying part of p'Czi an p'Czk, used in FE
local xi_hat 		"(xi - `pCi')"
local xk_hat		"(xk - `pCk')"
local x_hat			"(xi - `pC' - `rpDrp' )"
local x_hat_2nd		"(xi - `pC_2nd' - `rpDrp' )"

// construct structural vo
* let AVA be A'VA, and ensure its diagonals are positive.
local v0 "(0 "
forval j=1/`J' {
	local v0 "`v0' + p`j'*{AVA`j'`j'}*p`j'"
	
	local jplus1=`j'+1
	if `jplus1'<=`J' {
		forval k=`jplus1'/`J' {
			local v0 "`v0' + 2*p`j'*{AVA`j'`k'}*p`k'"
		}
	}
}

local v0 "`v0' )"

// make all equations, then put pieces together

* RE equations; first equation has first call to C, other equations use {Cj: } form
* note "-vj:", this is because we subtract the structural v0 term from E[q]
local j=1
												local eq`j'_RE 			"(eq`j'_RE: (p`j'*qi`j' - ((`Ak'*`Aikbar'     - 2*`x_hat'*`Aikbar'     + `x_hat_2nd'*`x_hat_2nd') * {d`j'}/`b_p' + (`x_hat_2nd'-`Aikbar')*{b`j'} + `Aikbar`j'' + `pC`j'' + `Drp`j'' - `v0'*{d`j'}/`b_p' ) ) )"
if noMeasError==1 			local eq`j'_RE 			"(eq`j'_RE: (p`j'*qi`j' - ((`Abar'*`Abar'     - 2*`x_hat'*`Abar'       + `x_hat_2nd'*`x_hat_2nd') * {d`j'}/`b_p' + (`x_hat_2nd'-`Abar'  )*{b`j'} + `Abar`j''   + `pC`j'' + `Drp`j'' - `v0'*{d`j'}/`b_p' ) ) )"
if `J'>2 {
	forval j=2/`Jm1' {
														local eq`j'_RE 	"(eq`j'_RE: (p`j'*qi`j' - ((`Ak'*`Aikbar' 		- 2*`x_hat_2nd'*`Aikbar' + `x_hat_2nd'*`x_hat_2nd') * {d`j'}/`b_p' + (`x_hat_2nd'-`Aikbar'  )*{b`j'} + `Aikbar`j'' + `pC`j'' + `Drp`j'' - `v0'*{d`j'}/`b_p' ) ) )"
		if noMeasError==1 			local eq`j'_RE 	"(eq`j'_RE: (p`j'*qi`j' - ((`Abar'*`Abar'  		- 2*`x_hat_2nd'*`Abar'   + `x_hat_2nd'*`x_hat_2nd') * {d`j'}/`b_p' + (`x_hat_2nd'-`Abar'    )*{b`j'} + `Abar`j''   + `pC`j'' + `Drp`j'' - `v0'*{d`j'}/`b_p' ) ) )"
	}
}

*	FE equations
forval j=1/`Jm1' {
										local eq`j'_FE "(eq`j'_FE: ((p`j'*qi`j' - p`j'*qk`j') - (`xi_hat'^2 - `xk_hat'^2) * {d`j'}/`b_p' - (`xi_hat' - `xk_hat')*({b`j'} - 2*(`Aikbar' + `pCg' + `rpDrp')*{d`j'}/`b_p') - (`pCi`j'' - `pCk`j'')			))"
	if noMeasError==1 local eq`j'_FE "(eq`j'_FE: ((p`j'*qi`j' - p`j'*qk`j') - (`xi_hat'^2 - `xk_hat'^2) * {d`j'}/`b_p' - (`xi_hat' - `xk_hat')*({b`j'} - 2*(`Abar'   + `pCg' + `rpDrp')*{d`j'}/`b_p') - (`pCi`j'' - `pCk`j'')			))"
}

// clean up equations, and combine
local eqs_RE ""
local eqs_FE ""
forval j=1/`Jm1' {
	local eq`j'_RE: subinstr local eq`j'_RE "( 0 +" "(", all
	local eq`j'_RE: subinstr local eq`j'_RE "( 0  +" "(", all
	local eq`j'_RE: subinstr local eq`j'_RE "  " " ", all
	local eq`j'_FE: subinstr local eq`j'_FE "( 0 +" "(", all
	local eq`j'_FE: subinstr local eq`j'_FE "( 0  +" "(", all
	local eq`j'_FE: subinstr local eq`j'_FE "  " " ", all
	local eqs_RE "`eqs_RE' `eq`j'_RE'"
	local eqs_FE "`eqs_FE' `eq`j'_FE'"
}

noi di "`eqs_RE'"
noi di "`eqs_FE'"

* replace the first occurrence of Ajj with the Alist
if Afull==0 {
	forval j=1/`J' {
		local eqs_RE=subinstr("`eqs_RE'","{A`j'`j':}","{A`j'`j':$Alist}",1)
		local eqs_FE=subinstr("`eqs_FE'","{A`j'`j':}","{A`j'`j':$Alist}",1)
	}
}
if Afull==1 {
	forval j=1/`J' {
		forval l=1/`J' {
			local eqs_RE=subinstr("`eqs_RE'","{A`j'`l':}","{A`j'`l':$Alist}",1)
			local eqs_FE=subinstr("`eqs_FE'","{A`j'`l':}","{A`j'`l':$Alist}",1)
		}
	}
}

if simple_model==1 {
	// construct z sums and interactions
	if `T'>0 {
		
		local RE_cz_2		 "+ {d}*(0"
		local RE_acz     ""
		local RE_acz1     ""
		local RE_acz2     ""
		local RE_acz_drv "" /// for calculating the derivative
		local RE_cz_cxz  ""
		local RE_czcz    ""
		local FE_czi     ""
		local FE_czk     ""
		local FE_czg     ""
		forv i=1/`T' {
			local RE_cz_2  "`RE_cz_2' + {C`i'}*zi`i'*p1"
			local RE_acz 				"`RE_acz'      + 2*{d}*{a} *{C`i'}*zi`i'*p1"
			local RE_acz1 			"`RE_acz2'     + 2*{d}*{a1}*{C`i'}*zi`i'*p1"
			local RE_acz2 			"`RE_acz2'     + 2*{d}*{a2}*{C`i'}*zi`i'*p1"
			local RE_cz_cxz "`RE_cz_cxz' + {C`i'}*zi`i'*p1 + 2*{d}*{b}*{C`i'}*xi*zi`i'*p1"
			
			forv j=1/`T' {
				local RE_czcz "`RE_czcz' + {d}*{C`i'}*{C`j'}*zi`i'*zi`j'"
			}
			
			if `i'<=`T' {
				local FE_czi "`FE_czi' + {C`i'}*zi`i'*p1"
				local FE_czk "`FE_czk' + {C`i'}*zk`i'*p1"
			}
			else {
				local FE_czg "`FE_czg' + {C`i'}*zi`i'*p1"
			}
		}
		local RE_cz_2 "`RE_cz_2')^2"
	}
	
	local eqs_RE "(eq1_RE: (p1*qi1-(({a}*p1*qikbar1*{a}*p1*qk1*{d}+( {a}     `RE_acz' +2*{a}*{b}*{d}*xi)*p1*qikbar1+(xi*{b} `RE_cz_cxz' +xi*xi*{b}*{b}*{d}) `RE_cz_2' +{v0}))))"
		
	local xi_hat "({b}*p1*xi `FE_czi')"
	local xk_hat "({b}*p1*xk `FE_czk')"
	
	local eqs_FE 												"(eq1_FE: (p1*(qi1-qk1) - (`xi_hat'^2 - `xk_hat'^2)*{d} - (`xi_hat' - `xk_hat')*(1 + 2*{d}*({a}*p1*qikbar1 `FE_czg' ))	))"
}

di "`eqs_RE'"
di "`eqs_FE'"

/******************************************************************************
 construct instruments
******************************************************************************/

//make squares and differences of x
tempvar x2i x2k ximxk x2imx2k xiP
g double `xiP'=xi*local_cpi
g double `x2i'=xi*xi
g double `x2k'=xi*xi
g double `ximxk'=xi-xk
g double `x2imx2k'=xi*xi-xk*xk

g xbarlag_x2barlag = xbarlag*x2barlag
g xbarlag2  = xbarlag^2
g x2barlag2 = x2barlag^2

//make interactions with z and zp
forval t=1/`T' {
	tempvar xizi`t' xkzk`t' xiPzi`t' zi`t'mzk`t' z2i`t'mz2k`t' xizi`t'mxkzk`t' zi`t'mzk`t'_xbarlag
	g double `xizi`t''=xi*zi`t'
	g double `xkzk`t''=xk*zk`t'
	g double `xiPzi`t''=xi*local_cpi*zi`t'
	g double `zi`t'mzk`t''=zi`t'-zk`t'
	g double `z2i`t'mz2k`t'' = zi`t'^2-zk`t'^2
	g double `zi`t'mzk`t'_xbarlag' = `zi`t'mzk`t''*xbarlag
	g double `xizi`t'mxkzk`t''=`xizi`t''-`xkzk`t''
	forval j=1/`J' {
		tempvar zi`t'p`j' zi`t'p`j'_sq xizi`t'p`j' zi`t'mzk`t'p`j'
		g double `zi`t'p`j''= zi`t'*p`j'
		g double `zi`t'p`j'_sq' = (zi`t'*p`j')^2
		g double `xizi`t'p`j''=xi*zi`t'*p`j'
		g double `zi`t'mzk`t'p`j''=`zi`t'mzk`t''*p`j'
		forv s=1/`T' {
			tempvar zi`t'zi`s'p`j'
			g double `zi`t'zi`s'p`j'' = zi`t'*zi`s'*p`j'*p`j'
		}
	}
} 

// make group-level instruments, and the instrument lists rg, xrg
if noMeasError==0	local rg "xbarlag"
if noMeasError==1	local rg "xbar"

local rg "`rg' x2barlag"
local rg "`rg' xbar2lag"
local rg "`rg' rootxbarlag"

local Tindivm1=`Tindiv'-1
// lag instruments
forval t=1/`Tindiv' {
		local rg "`rg' z`t'barlag"
} 

local zrg ""
forval t=1/`Tindiv' {
	foreach var of varlist `rg' {
		tempvar zi`t'`var'
		g double `zi`t'`var''=zi`t'*`var'
		local zrg "`zrg' `zi`t'`var''"
	}
}

local xrg ""
local prg ""
local xprg ""
foreach var of varlist `rg' {
	tempvar xi`var'
	g double `xi`var''=xi*`var'
	local xrg "`xrg' `xi`var''"	
	
	forval j=1/`J' {
		tempvar p`j'`var' xp`j'`var'
		g double `p`j'`var''=p`j'*`var'
		g double `xp`j'`var''=xi*p`j'*`var'
		local prg "`prg' `p`j'`var''"
		local xprg "`xprg' `xp`j'`var''"
	}
}

// make individual-level instruments: xi, xi2, pj, xipj, zitpJ, xizitpJ, round, 1 
// and individual-level equation specific instruments: rpjrpl, ztpj, round*pj
//   a longer individual-level list is: xi, xi2, pj, xipj, rpjrps, xirpjrps, zitpj, xzitpj
local budget "xi `x2i'"

local rootprootp ""
local xrootprootp ""
forval j=1/`J' {
	tempvar xip`j'
	g double `xip`j''=xi*p`j'
	local pinsts "`pinsts' p`j'"
	local xpinsts "`xpinsts' `xip`j''"
	forval s=`j'/`J' {
		tempvar rp`j'rp`s' xirp`j'rp`s'	
		g double `rp`j'rp`s''=sqrt(p`j')*sqrt(p`s')
		g double `xirp`j'rp`s''=xi*sqrt(p`j')*sqrt(p`s')
		local rootprootp "`rootprootp' `rp`j'rp`s'' "
		local xrootprootp "`xrootprootp' `xirp`j'rp`s''"
	}
} 
	
// demographics multiplied by prices and budget
local zinsts ""
local xzinsts ""
local xPzinsts ""
local pzinsts ""
local xpzinsts ""
forval t=1/`T' { 
	local zinsts "`zinsts' zi`t'"
	local xzinsts "`xzinsts' `xizi`t''"	
	local xPzinsts "`xPzinsts' `xiPzi`t''"
	forval j=1/`J' {
		local pzinsts "`pzinsts' `zi`t'p`j''"
		local xpzinsts "`xpzinsts' `xizi`t'p`j''"	
	}
} 

// price insts 
local roundinsts " "
forval j=1/`J' {
	forval k=`j'/`J' {
		tempvar p`j'p`k'
		g `p`j'p`k''=p`j'*p`k'
		local roundinsts "`roundinsts' `p`j'p`k''"
	}
}

// instruments for all equations are ri:  x,x2,p,z,zx, 
// instruments for each equation are rij: rootpjps,zpj,xzpj,roundpj
local qhat_insts ""
local qhat_insts "`qhat_insts' `zrg'"
local qhat_insts "`qhat_insts' `prg'"
local qhat_insts "`qhat_insts' `rg' "

local r_RE 										"`rootprootp' `budget' `pzinsts' "

local count: word count `r_RE' 
di "total instruments: `count'"

local qhat_hats ""
local pqhat ""
local xpqhat "" 
local pqhat_sq ""
forval j=1/`J' {
	if noMeasError==0 reg qikbar`j' `r_RE' `qhat_insts', vce(cluster group_round) 
	if noMeasError==1 reg qbar`j'   `r_RE' `qhat_insts', vce(cluster group_round) 

	tempvar qikhat`j' xiPqikbar`j' xiPqikhat`j' 
	predict `qikhat`j''
	summ `qikhat`j''
	
	// use all prices times all qikhats as instruments, interacted with xi
	forval s=1/`J' {
		tempvar  p`s'qikhat`j' xip`s'qikhat`j' p`s'qikhat`j'_sq
		g double `p`s'qikhat`j''=p`s'*`qikhat`j''	
		g double `xip`s'qikhat`j''=xi*p`s'*`qikhat`j''	
		g double `p`s'qikhat`j'_sq' = `p`s'qikhat`j''^2
		
		local pqhat "`pqhat' `p`s'qikhat`j''"
		local xpqhat "`xpqhat' `xip`s'qikhat`j''"				
		local pqhat_sq "`pqhat_sq' `p`s'qikhat`j'_sq'"
	}
}

local qhat_hats "`qhat_hats' `pqhat' `xpqhat'"

* interact qhat_hats with Alist
di "`qhat_hats'"
if "$Alist"~="one" {
	local qhat_hats_backup "`qhat_hats'"
	local qhat_hats ""
	foreach var1 of local qhat_hats_backup {
		foreach var2 of global Alist {
			g `var1'_`var2'=`var1'*`var2'
			sum `var1' `var2' `var1'_`var2'
			local qhat_hats "`qhat_hats' `var1'_`var2'"
		}
	}
}

local r_RE 	"`r_RE' `qhat_hats'"

/****************************************************************************
Set instruments and start values
****************************************************************************/

if simple_model==1	{
	local r_RE "xbarlag `xixbarlag' `budget'"
	local r_RE1 ""
	
	forv i=1/`T' {
		local r_RE "`r_RE' zi`i'"
	}

	local initial_values_RE "a -0.6674374  b .3126426  d  0.6948198  v0   -0.0063737  "
}

if `J'==2 & simple_model==0	{
	g y = p1*qi1
	g ybar_q1 = p1*qikbar1 
	g ybar_q2 = p2*qikbar2 
	g ybar = p1*qikbar1 + p2*qikbar2
	g ybar2 = ybar^2
	g ybar_q1_2 = ybar_q1^2
	g ybar_q2_2 = ybar_q2^2
	g ybar_q1_q2 = ybar_q1*ybar_q2
	g ybarx = ybar*xi
	g ybarx_1 = ybar_q1*xi
	g ybarx_2 = ybar_q2*xi
	g xi2 = xi^2
	
	forv j = 1(1)`T' {
		g pzi`j' = p1*zi`j'
	}
	
	if same_spillover==1 {
		reg y ybar2 ybarx xi2 ybar xi ybar_q1 pzi*
		local b = _b[xi]
		local a = _b[ybar_q1]
		if same_spillover==1 {
			local a = logit(_b[ybar_q1]/2 + .5)
			if `a'==. local a = .5
		}
		local d = _b[xi2]
		local initial_values_RE "A11:one `a' b1 `b' d1 `d'"
	}
	else {
		reg y xi2 ybar_q1_2 ybar_q2_2 ybarx_1 ybarx_2 ybar_q1_q2 xi ybar_q1 ybar_q2 pzi*
		local b = _b[xi]
		local a1 = _b[ybar_q1]/(1-`b')
		local a2 = -_b[ybar_q2]/`b'
		if same_spillover==1 {
			local a1 = logit(_b[ybar_q1]/)2*(1-`b')) + .5)
			local a2 =  logit(-_b[ybar_q2]/(2*`b') + .5)
			
			if `a1'==. local a1 = .5
			if `a2'==. local a2 = .5			
		}
		local d = _b[xi2]

		local initial_values_RE "A11:one `a1' A22:one `a2' b1 `b' d1 `d'"
	}
	
	drop y ybar ybar2 ybarx xi2 ybar_q1 ybar_q2 pzi* ybar_q1_2 ybar_q2_2 ybar_q1_q2
	forv j = 1(1)`T' {
		local coef = _b[pzi`j']
		local initial_values_RE "`initial_values_RE' C1:zi`j' `coef'"
	}
}

if `J'==3 & simple_model==0											local initial_values_RE "A11:one 0.6 A22:one 0.6 A33:one 0.6 b1 0.01 b2 0.26 d1 0.03 d2 0.01"
if `J'==3 & simple_model==0 & same_spillover==1 local initial_values_RE "A11:one 0.6 b1 0.01 b2 0.26 d1 0.03 d2 0.01"
if `J'==4 & simple_model==0											local initial_values_RE "A11:one 0.6 A22:one 0.6 A33:one 0.6 A44:one 0.6 b1 -0.08 b2 0.24 b3 0.24 d1 0.01 d2 0.01 d3 -0.01"
if `J'==4 & simple_model==0 & same_spillover==1 local initial_values_RE "A11:one 0.6 b1 -0.08 b2 0.24 b3 0.24 d1 0.01 d2 0.01 d3 -0.01"

if simple_model==0 local initial_values_RE "`initial_values_RE' AVA11:_cons 0.1 AVA22:_cons 0.2"

if `J'==2 & estimate_RE==1 {
	
	capture noisily gmm `eqs_RE' $GMM_weight, ///
		instruments(`r_RE') $trace_level  ///
		$derivatives   ///
		vce(clust ${clust_var}) $wmatrix  $w_initial from(`initial_values_RE') $tol_level $maxiter $technique $technique_RE
	
	if simple_model==1 capture noisily nlcom _b[/a]+_b[/c]+2*_b[/a]*_b[/d]*(0.3*_b[/a]+1.19*_b[/b]) 
	if simple_model==1 capture noisily nlcom _b[/a]+2*_b[/a]*_b[/d]*(0.3*_b[/a]+1.19*_b[/b])
	
	if simple_model==1 {
		// calculate derivative and save
		local RE_acz_drv ""
		tempvar junk
		g `junk'=.
		forv i=1/`Tm1' {
			replace `junk' = zi`i'*p1
			su `junk' $GMM_weight
			local mean_zi`i'_p1 = r(mean)
			local RE_acz_drv 	"`RE_acz_drv' + _b[/C`i']*`mean_zi`i'_p1'"
		}
		replace `junk'=p1*qikbar1
		su `junk' $GMM_weight
		local mean_y = r(mean)
		su xi $GMM_weight
		local mean_x = r(mean)
	
		local c_term ""
		
		noi di "_b[/a] `c_term' + 2*_b[/a]*_b[/d]*(`mean_x'*_b[/b]+`mean_y'*_b[/a] `RE_acz_drv')"
		estimates store gmm_est
		nlcom _b[/a]  `c_term' + 2*_b[/a] *_b[/d]*(`mean_x'*_b[/b]+`mean_y'*_b[/a]  `RE_acz_drv'), post
		estimates save "${filename_RE}_dF", replace 
		estimates restore gmm_est
	}
}

if `J'==3 & estimate_RE==1 {
	capture noisily gmm `eqs_RE', ///
		instruments(1:`r_RE' `r_RE1') instruments(2:`r_RE' `r_RE2')  ///
		$derivatives $trace_level  /// 
		vce(clust ${clust_var}) $wmatrix  $w_initial from(`initial_values_RE')  $tol_level $maxiter $technique
}
if `J'==4 & estimate_RE==1 {
	capture noisily gmm `eqs_RE' $GMM_weight, ///
		instruments(1:`r_RE' `r_RE1') instruments(2:`r_RE' `r_RE2') instruments(3:`r_RE' `r_RE3') ///
		$derivatives $trace_level  ///
		vce(clust ${clust_var}) $wmatrix  $w_initial from(`initial_values_RE')  $tol_level $maxiter $technique
}

// save coefs
matrix est_b=e(b)
unique hhidi
local N_hh = r(sum)
estadd scalar N_hh = r(sum)
unique group_round
estadd scalar N_grp = r(sum)
estadd scalar avg_grp_size = `N_hh'/r(sum)	
if estimate_RE==1 {
	estimates save "$filename_RE", replace 
	estimates save temp1_RE, replace
}

estimates store temp1_FE

/*****************************************************************************
Set starting values and instruments for FE analysis
*****************************************************************************/

if simple_model==0 {
	
	// make FE instruments
	
	// denoting ximxk as x and zimzk of individual zs as zi:
	//   x,x2,p,zi,px,zi,pzi
	// local xdiff "`ximxk' `x2imx2k' `xixk'"
	local xdiff "`ximxk' "
	local xdiffxdiff ""
	local zindivdiff ""
	local pzindivdiff ""
	local pzindivdiffpqhat ""
	local pxzindivdiff ""
	local p2zindiv2diff ""
	local zg ""
	local pzg ""
	local zindivdiffxdiff ""
	local pzindivdiffxdiff ""
	local pzindivdiffp ""
	local pzindivdiffpzg ""
	local zgxdiff ""
	local pzgxdiff ""
	tempvar ximxkxdiff x2imx2kxdiff xixkxdiff
	g double `ximxkxdiff'=`ximxk'*`ximxk'
	g double `x2imx2kxdiff'=`x2imx2k'*`ximxk'
	g double `xixkxdiff'=xi*xk*`ximxk'

	local xdiffxdiff "`x2imx2k'"
	
	forval t=1/`Tindiv' {
		tempvar zi`t'mzk`t'xdiff
		g double `zi`t'mzk`t'xdiff'=`zi`t'mzk`t''*`ximxk'
		local zindivdiff "`zindivdiff' `zi`t'mzk`t'' "
		local zindivdiffxdiff "`zindivdiffxdiff' `zi`t'mzk`t'xdiff' p`j'2z`t'2diff "
		forval j=1/`J' {
			tempvar zi`t'mzk`t'p`j'xdiff xizi`t'mxkzk`t'p`j' p`j'2z`t'2diff 
			g double `zi`t'mzk`t'p`j'xdiff'=`zi`t'mzk`t'p`j''*`ximxk'
			g double `xizi`t'mxkzk`t'p`j''=(xi*zi`t'-xk*zk`t')*p`j'
			g double `p`j'2z`t'2diff'=(p`j')^2*((zi`t')^2-(zk`t')^2)
			local pzindivdiff "`pzindivdiff'  `zi`t'mzk`t'p`j'' "
			local pzindivdiffxdiff "`pzindivdiffxdiff'  `zi`t'mzk`t'p`j'xdiff' "
			local pxzindivdiff "`pxzindivdiff' `xizi`t'mxkzk`t'p`j'' "
			local p2zindiv2diff "`p2zindiv2diff' `p`j'2z`t'2diff'"
			forval l=1/`J' {
				tempvar zdiff`t'p`j'p`l' 
				g double `zdiff`t'p`j'p`l''=`zi`t'mzk`t'p`j''*p`l'
				local pzindivdiffp "`pzindivdiffp' `zdiff`t'p`j'p`l''"
				foreach name of global Alist {
					tempvar  zdiff`t'p`j'p`l'q`l'`name'   
					g double `zdiff`t'p`j'p`l'q`l'`name''=`zi`t'mzk`t'p`j''*`p`l'qikhat`l''*`name'
					local pzindivdiffpqhat "`pzindivdiffpqhat' `zdiff`t'p`j'p`l'q`l'`name''"
				}
				if `Tindivp1'<=`T' {
					forval s=`Tindivp1'/`T' {
						tempvar zdiff`t'p`j'zi`s'p`l'
						* don't generate these to save memory!
						gen double `zdiff`t'p`j'zi`s'p`l''=	`zdiff`t'p`j'p`l''*zi`s'
						local pzindivdiffpzg " `pzindivdiffpzg' `zdiff`t'p`j'zi`s'p`l'' "
					}
				}
			}
		}
	}
	if `Tindivp1'<=`T' {
		forval t=`Tindivp1'/`T' {
			tempvar zi`t'xdiff
			g double `zi`t'xdiff'=zi`t'*`ximxk'
			local zg "`zg' zi`t'"
			forval j=1/`J' {
				tempvar zi`t'p`j'xdiff
				g double `zi`t'p`j'xdiff'=`zi`t'p`j''*`ximxk'
				local pzg "`pzg' `zi`t'p`j''"
				local pzgxdiff "`pzgxdiff' `zi`t'p`j'xdiff'"
			}
		}
	}		
	 	
	// pzgxdiff for group-varying parts of C, rootpxdiff for R, pqhatxdiff for A
	// denoting zg as group-zs
	//  zg,pzgx,rprp,rprpx
	local rootprootp ""	
	local rootprootpxdiff ""
	
	forval j=1/`J' {
		forval s=`j'/`J' {
			tempvar xdiffrp`j'rp`s'
			g double `xdiffrp`j'rp`s''=`ximxk'*sqrt(p`j')*sqrt(p`s')
			
			if (`s'==`j')		local rootprootp "`rootprootp' `rp`j'rp`s'' "
			
			local rootprootpxdiff "`rootprootpxdiff' `xdiffrp`j'rp`s'' "			
		}
	}

	local pqhatxdiff ""
	forval j=1/`J' {
		forval s=1/`J' {
			foreach name of global Alist {
				tempvar   p`s'qikhat`j'xdiff`name'
				g double `p`s'qikhat`j'xdiff`name''=`ximxk'*p`s'*`qikhat`j''*`name'
				
				local pqhatxdiff "`pqhatxdiff' `p`s'qikhat`j'xdiff`name'' "		
			}	
		}
	}
	
	// set instruments
	local r_FE "`xdiff' `pzindivdiff' `xdiffxdiff' `pzgxdiff' `pqhatxdiff' `pzindivdiffpqhat' `pxzindivdiff'"
}


if simple_model==1 {
	foreach var in xbarlag x2barlag xbar2lag rootxbarlag {
		tempvar ximxk`var'
		g `ximxk`var''=`ximxk'*`var'
	}
	
	/******************
	Initial values
	******************/
	
	if init_FE_from_RE==1 {
		estimates use "$filename_RE"
		
		// clear init values
		local initial_values_FE ""
		
		// extract vector of coefs and paramter names from 
		matrix coefs = e(b)
		local paramlist = e(params)
		
		// iterate through paramter name list, taking 
		local t=0
		foreach p of local paramlist {
			local `++t'
			local p_nice = subinstr("`p'",":_cons","",.)
			local est = coefs[1,`t']
			if "`p_nice'"!="c" & "`p_nice'"!="v0" local initial_values_FE "`initial_values_FE' `p_nice' `est'"
		}		
	}
	
	// make z diff instruments
	local pzindivdiff ""
	local pz2indivdiff ""
	local pzXzindivdiff ""
	local pxzindivdiff ""
	
	forv i=1/`T' {
		
		tempvar z2i`i'mz2k`i'p1p1
		g `z2i`i'mz2k`i'p1p1' = `z2i`i'mz2k`i''*p1*p1
		
		local pzindivdiff "`pzindivdiff' `zi`i'mzk`i'p1' "
		local pz2indivdiff "`pz2indivdiff' `z2i`i'mz2k`i'p1p1'"
		
		tempvar xizi`i'mxkzk`i'p1
		g double `xizi`i'mxkzk`i'p1'=(xi*zi`i'-xk*zk`i')*p1*p1
		local pxzindivdiff "`pxzindivdiff' `xizi`t'mxkzk`t'p`j'' "
					
		forv j=1(1)`T' {
			if `j'>`i' {
				tempvar zi`j'zi`i'mzkz`j'k`i'p1
				g double `zi`j'zi`i'mzkz`j'k`i'p1'=(zi`j'*zi`i'-zk`j'*zk`i')*p1*p1 
				local pzXzindivdiff "`pzXzindivdiff' `zi`j'zi`i'mzkz`j'k`i'p1'"
			}
		}
	}

	// p z interactions 
	local pzg ""
	local pzgxdiff ""	
	if `Tindivp1'<=`T' {
		forval t=`Tindivp1'/`T' {
			tempvar p1xdiffz`t' 
			g double `p1xdiffz`t''=p1*`ximxk'*zi`t'
			local pzgxdiff "`pzgxdiff' `p1xdiffz`t''"
			local pzg "`pzg' `zi`t'p1'"
		}
	}

	tempvar p1ximxk  p1x2imx2k
	g `p1ximxk' = p1*`ximxk'
	g `p1x2imx2k' = (p1^2)*`x2imx2k'
	
	// define instruments
	local r_FE "`p1ximxk' `pzindivdiff' `p1x2imx2k'  `pz2indivdiff' `pzgxdiff'     `pxzindivdiff' `pzXzindivdiff'"
	
	// add prediction of quantity as extra instrument
	local pqhat ""
	local qhat ""
	local pqhatxdiff ""
	local pqhatxdiff2 ""
	local pzindivdiffpqhat ""
	local pzindivdiffpxbar_lag ""

	tempvar qik_hat1
	 reg qikbar1 	`r_RE' `qhat_insts', vce(cluster group_round) 
	predict `qik_hat1'
	local qhat "`qhat' `qik_hat1'"
	
	tempvar p1_qikbar1 p1_qikhat1 p1qikbar1xdiff p1qikhat1xdiff p1qikbar1xdiff2 p1qikhat1xdiff2
	
	g double `p1_qikbar1' = p1*qikbar1
	reg `p1_qikbar1' 	`r_FE' `qhat_insts', vce(cluster group_round) 
	predict `p1_qikhat1'
	local pqhat "`pqhat' `p1_qikhat1'"
	
	g double `p1qikbar1xdiff' = p1*qikbar1*`p1ximxk'
	g `p1qikhat1xdiff' = `p1_qikhat1'*`p1ximxk'
	
	local pqhatxdiff "`pqhatxdiff' `p1qikhat1xdiff'"
	
	g double `p1qikbar1xdiff2'=p1*qikbar1*`x2imx2k'
	g `p1qikhat1xdiff2' = `p1_qikhat1'*`x2imx2k'
	local pqhatxdiff2 "`pqhatxdiff2' `p1qikhat1xdiff2'"		
	
	forv t=1/`Tindiv' {
		tempvar zdiff`t'p1p1qhat zdiff`t'p1p1qbar pzdiff`t'_xbarlag
		g `zdiff`t'p1p1qbar' = (zi`t'-zk`t')*p1*p1*qikbar1
		g `zdiff`t'p1p1qhat' = (zi`t'-zk`t')*p1*`p1_qikhat1'
		local pzindivdiffpqhat "`pzindivdiffpqhat' `zdiff`t'p1p1qhat'"
		
		g `pzdiff`t'_xbarlag' = (zi`t'-zk`t')*p1*xbarlag
		local pzindivdiffpxbar_lag "`pzindivdiffpxbar_lag' `pzdiff`t'_xbarlag'"
	}
	
	local r_FE "`r_FE' `pqhatxdiff'   `pzindivdiffpqhat' `pqhat' "		
}

// starting values
if init_FE_from_RE==1 & "$init_FE_file"!="" {
	estimates use "$init_FE_file"
	local initial_values_FE ""
	
	// extract vector of coefs and paramter names from 
	matrix coefs = e(b)
	local paramlist = e(params)
	
	// iterate through paramter name list, taking 
	local t=0
	foreach p of local paramlist {
		local `++t'
		local p_nice = subinstr("`p'",":_cons","",.)
		if substr("`p_nice'",1,1)!="A" local p_nice = subinstr("`p_nice'",":zi","",.)
		local est = coefs[1,`t']
		if "`p_nice'"!="c" & "`p_nice'"!="v0" & regexm("`p_nice'","AVA")==0 local initial_values_FE "`initial_values_FE' `p_nice' `est'"
	}	
}

if estimate_FE==1 {	
	capture noisily gmm `eqs_FE' $GMM_weight , instruments(`r_FE') $derivatives $trace_level $tol_level $maxiter ///
		vce(clust ${clust_var}) $wmatrix  $w_initial from(`initial_values_FE')  $technique
	
	estat overid
	di "$S_DATE $S_TIME"
	unique hhidi
	local N_hh = r(sum)
	estadd scalar N_hh = r(sum)
	unique group_round
	estadd scalar N_grp = r(sum)	
	estadd scalar avg_grp_size = `N_hh'/r(sum)	
	estimates save "$filename_FE", replace
	
	if simple_model==1 {
		local RE_acz_drv ""
		forv i=1/`Tindiv' {
			cap drop junk
			g junk = zi`i'*p1
			su junk $GMM_weight
			local mean_zi`i'_p1 = r(mean)
				local RE_acz_drv 	"`RE_acz_drv' + _b[/C`i']*`mean_zi`i'_p1'"
		}
		noi di "`RE_acz_drv'"
		
		cap g junk=.
		replace junk=p1*qikbar1
		su junk $GMM_weight
		local mean_y = r(mean)
		su xi $GMM_weight
		local mean_x = r(mean)
		
		estimates store gmm_est
		nlcom _b[/a]  + 2*_b[/a] *_b[/d]*(`mean_x'*_b[/b]+`mean_y'*_b[/a]  `RE_acz_drv'), post
		estimates save "${filename_FE}_dF", replace 
		estimates restore gmm_est
	}
}	

